from typing import Any

def mk_gen(): ...
def mk_awaitable(): ...
def mk_coroutine(): ...

Generator = ...  # type: Any
Awaitable = ...  # type: Any
Coroutine = ...  # type: Any

def isawaitable(obj): ...

PATCHED = ...  # type: Any

def patch(patch_inspect: bool = ...): ...
